from storedoc.services import SUPPORTED_SERVICES
from storedoc.local import LocalStorage


class Service(object):

    def __init__(self, service_name):
        if service_name in SUPPORTED_SERVICES:
            self.service_name = service_name
        else:
            raise Exception('Service not found.')


    def available_services(self):
        return dict(SUPPORTED_SERVICES.keys())


    def describe_all_services(self):
        print('Services: ')
        for service_name in SUPPORTED_SERVICES:
            description = SUPPORTED_SERVICES[service_name]['description'] 
            print('{} - {} '.format(service_name, description)) 


    def get_conn(self):
        return getattr(self, 'service', 'Service not connected.')


    def describe(self):
        return SUPPORTED_SERVICES[self.service_name]['description']


    def connect(self, **credentials):
        self.service = SUPPORTED_SERVICES[self.service_name]['service'](**credentials)


    def upload_file(self, file, **kwargs):
        return self.service.upload_file(file, **kwargs)



from storedoc.local import LocalStorage
from storedoc.s3 import S3Service
from storedoc.spaces import DOService


SUPPORTED_SERVICES = {
    'local': {
        'service': LocalStorage,
        'description': 'Store locally'
    },
    's3': {
        'service': S3Service,
        'description': 'Amazon S3 or Amazon Simple Storage Service is\
a "simple storage service" offered by Amazon Web Services that provides object storage through a web service interface.'
    },
    'aws': {
        'service': S3Service,
        'description': 'Amazon S3 or Amazon Simple Storage Service is\
a "simple storage service" offered by Amazon Web Services that provides object storage through a web service interface.'
    },
    'spaces': {
        'service': DOService,
        'description': 'DigitalOcean Spaces are ideal for storing static, unstructured\
data like audio, video, and images as well as large amounts of text'
    },
    'do': {
        'service': DOService,
        'description': 'DigitalOcean Spaces are ideal for storing static, unstructured\
data like audio, video, and images as well as large amounts of text'
    }
}


from storedoc.s3 import S3Service

class DOService(S3Service):

	def __init__(self, region_name, endpoint_url, **credentials):
		endpoint_url = 'https://{}.digitaloceanspaces.com'.format(region_name)
		super(DOService, self).__init__(region_name, endpoint_url, **credentials)
		self.base_url = endpoint_url
		self.description = 'DigitalOcean Spaces are ideal for storing static, unstructured\
data like audio, video, and images as well as large amounts of text'
		



import os

class LocalStorage(object):

    def _get_filename(self, file):
        *_, filename =  file.filename.split('/')
        return filename


    def save_file(self, file, folder=''):
        filename = self._get_filename(file)
        media_location = os.path.join(os.getcwd(), filename)
        if folder and not os.path.exists(folder):
            os.mkdir(folder)
        if folder:
            media_location = os.path.join(os.path.abspath(
                os.path.join(os.getcwd(), folder)
            ), filename)
        file.save(media_location)
        return media_location



import os
import re

import boto3
import urllib
import mimetypes

from hashlib import md5
from time import localtime

from werkzeug.datastructures import FileStorage 


class S3Service(object):
    data_uri_pat = r'^data:(?P<content_type>[A-Za-z\/\-\+.]*);base64,(?P<bytes>.*)$'    

    def __init__(self,
            region_name,
            endpoint_url,
            aws_access_key_id,
            aws_secret_access_key):
        self.description = 'Amazon S3 or Amazon Simple Storage Service is\
a "simple storage service" offered by Amazon Web Services that provides object storage through a web service interface.'
        self.conn = boto3.resource('s3',
                        region_name=region_name,
                        endpoint_url=endpoint_url,
                        aws_access_key_id=aws_access_key_id,
                        aws_secret_access_key=aws_secret_access_key)

    def _upload_file_to_cloud(self,
            file,
            file_key,
            bucket,
            mime,
            guess_mime,
            acl):
        buck = self.conn.Bucket(bucket)
        params = dict(
            Key=file_key,
            Body=file,
            ACL=acl)
        if mime:
            params['ContentType'] = mime
        elif guess_mime:
            guess_mime = mimetypes.guess_type(file_key)[0]
            if guess_mime:
                params['ContentType'] = guess_mime

        buck.put_object(**params)
        return '{}/{}/{}'.format(self.base_url, bucket, file_key)


    def _is_data_uri(self, file):
        return bool(re.match(self.data_uri_pat, file))

    def _get_file_bytes(self, data_uri):
        match = re.match(self.data_uri_pat, data_uri)
        if match:
            return match.groups()
        raise ValueError('Invalid file provided')

    def _get_filename(self, file):
        *_, filename =  file.filename.split('/')
        return filename

    def _get_random_filename(self, content_type):
        ext = mimetypes.guess_extension(content_type)
        filename = '{}{}'.format(
                md5(str(localtime()).encode('utf-8')).hexdigest(),
                ext
            )
        return filename

    def upload_file(self,
            file,
            bucket,
            folder='',
            filename=None,
            mime=None,
            guess_mime=False,
            acl='private'):
        if type(file) == FileStorage:
            _filename = self._get_filename(file)
        elif type(file) == str and self._is_data_uri(file):
            mime, file_bytes = self._get_file_bytes(file)
            _filename = self._get_random_filename(mime)
            response = urllib.request.urlopen(file)
            file = response.file.read()
        else:
            raise ValueError('Invalid file provided')

        filename = filename if filename else _filename
        file_key = filename
        if folder:
            file_key = '{}/{}'.format(folder, filename)

        file_url = self._upload_file_to_cloud(
            file,
            file_key,
            bucket,
            mime,
            guess_mime,
            acl
        )
        return file_url



__version__ = '0.1.8'

from setuptools import setup

from os import path

cwd = path.abspath(path.dirname(__file__))

with open(path.join(cwd, 'README.md'), 'r') as f:
    long_description = f.read()


setup(
    name='storedoc',
    version=__version__,
    description='A python library to upload files on cloud. Supported services - AWS S3, DigitalOcean Spaces, LocalStorage.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Rakesh Gunduka',
    author_email='rakesh.gunduka@gmail.com',
    install_requires=['boto3'],
    packages=['storedoc'],
    package_data={},
    license='http://www.opensource.org/licenses/mit-license.php',
    keywords='boto3 aws s3 digitalocean spaces',
    scripts=[
        'storedoc/local.py', 'storedoc/services.py',
        'storedoc/s3.py', 'storedoc/spaces.py'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent'
    ]
)
